#include <compress.h>

int compress(char * nom_fichier, char * nom_compresse, 
	int (*ecrire_entier)(FileBAB * fichier, unsigned int n), int verbose)
{
    FileBAB * compresse = NULL;
    Mots * mots = NULL;
    liste lmots = NULL, lponctu = NULL;
    cellule * c = NULL;
    char * mot = NULL;
    int position = 0, pos_fichier, lterminal;
    char ponctuation;
    struct stat s;
    int vaffiche = 0;

    /* stat() donne la taille du fichier en octets dans la structure stat */
    stat(nom_fichier, &s);
    pos_fichier = 0;
    lterminal = largeur_terminal();

    /* Fichier compress */
    compresse = ouvrir(nom_compresse, "w");
    if ( compresse == NULL )
	return 0;

    /* Lecture */
    mots = init_parseur(nom_fichier);
    if ( mots == NULL ) {
	fermer(compresse);
	return 0;
    }

    /* criture de la taille du fichier original */
    ecrire_entier(compresse, (int) s.st_size);

    /* Compression */
    while ( ( ponctuation = lire_mot(mots,&mot)) ) {
	if ( verbose 
	    && ((int)((pos_fichier/(float)s.st_size)*100)) == vaffiche ) {
	    vaffiche += 10;
	    puts("Mots :");
	    afficher_liste(lmots);
	    puts("Ponctuation : ");
	    afficher_liste_c(lponctu);
	}

	pos_fichier += strlen(mot)+1;

	/* Ajout de mot */
	c = recherche(lmots, mot, &position);
	if ( ! ecrire_entier(compresse, position) )
	    return 0;
	if ( position == 0 ) {
	    ajouter_fin_liste(&lmots, mot);
	    if ( ! ecrire(compresse, mot, 8*(strlen(mot)+1)) )
		return 0;
	} else
	    met_en_tete(&lmots, c);

	/* Ajout de ponctuation */
	c = recherche_c(lponctu, ponctuation, &position);
	if ( ! ecrire_entier(compresse, position) )
	    return 0;
	if ( position == 0 ) {
	    ajouter_fin_liste_c(&lponctu, ponctuation);
	    if ( ! ecrire_car(compresse, ponctuation) )
		return 0;
	} else
	    met_en_tete(&lponctu, c);

	/* le pos_fichier%20 sert  ralentir l'affichage */
	if ( ! verbose && (pos_fichier%20) == 0 )
	    progressbar(pos_fichier, (int) s.st_size, lterminal);
    }

    detruire_liste(lmots);
    detruire_liste(lponctu);
    fin_parseur(mots);
    fermer(compresse);
    
    return 1;
}

int decompress(char * nom_compresse, char * nom_fichier,
	int (*lire_entier)(FileBAB * fichier, unsigned int * n), int verbose)
{
    FileBAB * compresse = NULL;
    FILE * fichier = NULL;
    liste lmots = NULL, lponctu = NULL;
    cellule * c = NULL;
    char * mot = NULL;
    int pos = 0, taille = 0, taille_fichier, pos_fichier = 0, lterminal;
    char ponctuation;
    int vaffiche = 0;

    lterminal = largeur_terminal();

    /* Fichier compress */
    compresse = ouvrir(nom_compresse, "r");
    if ( compresse == NULL )
	return 0;

    /* Fichier dcompress */
    fichier = fopen(nom_fichier, "w");
    if ( fichier == NULL ) {
	fermer(compresse);
	return 0;
    }

    /* Lecture de la taille du fichier original */
    if ( ! lire_entier(compresse, &taille_fichier) )
	return 0;

    /* Dcompression */
    while ( lire_entier(compresse, &pos) && pos_fichier < taille_fichier ) {
	if ( verbose 
	    && ((int)((pos_fichier/(float)taille_fichier)*100)) == vaffiche ) {
	    vaffiche += 10;
	    puts("Mots :");
	    afficher_liste(lmots);
	    puts("Ponctuation : ");
	    afficher_liste_c(lponctu);
	}

	/* Lecture d'un mot */
	if ( pos == 0 ) {
	    lirechaine(compresse, &mot, &taille);
	    fprintf(fichier, "%s", mot);
	    ajouter_fin_liste(&lmots, mot);
	    pos_fichier += (taille-1);
	} else {
	    c = position(lmots, pos);
	    fprintf(fichier, "%s", c->mot);
	    met_en_tete(&lmots, c);
	    pos_fichier += strlen(c->mot);
	}

	/* Lecture d'une ponctuation */
	if ( ! lire_entier(compresse, &pos) ) {
	    fermer(compresse);
	    fclose(fichier);
	    detruire_liste(lmots);
	    detruire_liste(lponctu);
	    return 0;
	}

	if ( pos == 0 ) {
	    lire_car(compresse, &ponctuation);
	    fprintf(fichier, "%c", ponctuation);
	    ajouter_fin_liste_c(&lponctu, ponctuation);
	} else {
	    c = position(lponctu, pos);
	    fprintf(fichier, "%c", c->mot[0]);
	    met_en_tete(&lponctu, c);
	}

	pos_fichier++;

	/* pos_fichier%20 sert  ralentir l'affichage */
	if ( ! verbose && (pos_fichier%20) == 0 )
	    progressbar(pos_fichier, taille_fichier, lterminal);
    }

    free(mot);
    fermer(compresse);
    fclose(fichier);
    detruire_liste(lmots);
    detruire_liste(lponctu);
    
    return 1;
}
