#include <ecriture.h>

/* crit un bit dans le fichier 
 * L'criture relle ne se fait que quand l'octet de tampon est complet 
 */
int ecrirebit(FileBAB * fichier, char bit)
{
    fichier->tampon |= (bit << fichier->bit);
    fichier->bit++;
    if ( fichier->bit == 8 ) {
	if ( fwrite(&(fichier->tampon), sizeof(char), 1, fichier->fichier)
		<= 0 )
	    return 0;
	fichier->bit = 0;
	fichier->tampon = 0;
    }

    return 1;
}

/* 
 * Lit un bit dans le fichier
 */
int lirebit(FileBAB * fichier, char * bit)
{
    if ( fichier->bit == 0 )
	if ( fread(&(fichier->tampon), sizeof(unsigned char), 1, 
		    fichier->fichier) <= 0 ) 
	    return 0;

    *bit = ( fichier->tampon & (1 << fichier->bit) ) ? 1 : 0;

    fichier->bit = (fichier->bit+1) % 8;

    return 1;
}

/* 
 * Ouvrir un fichier
 */
FileBAB * ouvrir(char * nom_fichier, char * mode)
{
    FileBAB * fichier = malloc(sizeof(FileBAB));
    
    if ( fichier == NULL )
	return NULL;

    fichier->fichier = fopen(nom_fichier, mode);
    if ( fichier->fichier == NULL ) {
	free(fichier);
	return NULL;
    }

    fichier->bit = 0;
    fichier->tampon = 0;
    
    fichier->mode = malloc(strlen(mode)+1);
    if ( fichier->mode == NULL ) {
	fclose(fichier->fichier);
	free(fichier);
	return NULL;
    }
    strcpy(fichier->mode, mode);

    return fichier;
}

/* 
 * Ferme un fichier, crit avant le contenu du tampon en le compltant
 * si ncessaire
 */
void fermer(FileBAB * fichier)
{
    /* Vide le tampon */
    if ( fichier->mode[0] == 'w' )
	while ( fichier->bit != 0 ) {
	    ecrirebit(fichier, 0);
	}

    fclose(fichier->fichier);

    free(fichier->mode);
    free(fichier);
}

/* 
 * crit nbbits de la zone pointe par ptr
 */
int ecrire(FileBAB * fichier, void * ptr, int nbbits)
{
    if ( fichier->mode[0] != 'w' )
	return 0;

    int i;

    for ( i=0 ; i < nbbits ; i++ ) {
	if ( ((unsigned char *)ptr)[i/8] & ( 1 << (i%8) ) ) {
	    if ( ! ecrirebit(fichier, 1) )
		return 0;
	} else
	    if ( ! ecrirebit(fichier, 0) )
		return 0;
    }

    return 1;
}

/* 
 * crit un char dans le fichier
 */
int ecrire_car(FileBAB * fichier, char c)
{
    if ( fichier->mode[0] != 'w' )
	return 0;

    int i;

    for ( i=0 ; i < 8 ; i++ ) {
	if ( c & ( 1 << i ) ) {
	    if ( ! ecrirebit(fichier, 1) )
		return 0;
	} else
	    if ( ! ecrirebit(fichier, 0) )
		return 0;
    }

    return 1;
}

/* 
 * Lit une chaine de caractres dans un fichier de type FileBAB
 */
int lirechaine(FileBAB * fichier, char ** chaine, int * taille)
{
    if ( fichier->mode[0] != 'r' )
	return 0;

    if ( *taille == 0 ) {
	*chaine = calloc(10, sizeof(char));
	if ( *chaine == NULL )
	    return 0;
	*taille = 10;
    }

    char tmp = 0, bit = 0;
    int i = 0, j;
    do {
	tmp = 0;
	for ( j=0 ; j < 8 ; j++ ) {
	    if ( ! lirebit(fichier, &bit) )
		return 0;
	    if ( bit )
		tmp |= ( 1 << j );
	}
	(*chaine)[i] = tmp;
	i++;
	if ( i >= *taille ) {
	    *chaine = realloc(*chaine, *taille + 10);
	    *taille += 10;
	}
    } while ( tmp != 0 );

    *chaine = realloc(*chaine, (*taille = strlen(*chaine)+1));

    return 1;
}

/*
 * Lit un caractre dans un fichier de type FileBAB
 */
int lire_car(FileBAB * fichier, char * car)
{
    if ( fichier->mode[0] != 'r' )
	return 0;

    int i;
    char bit;

    *car = 0;
    for ( i=0 ; i < 8 ; i++ ) {
	if ( ! lirebit(fichier, &bit) )
	    return 0;
	if ( bit )
	    *car |= ( 1 << i );
    }

    return 1;
}

/*
 * Retourne le nombre de bits "utiles" dans un entier non sign
 */
unsigned int nbbits(unsigned int n)
{
    int nb;
    for ( nb=31 ; nb >= 0 ; nb-- )
	if ( n & ( 1 << nb ) )
	    return nb+1;
    return 1;
}


/* 
 * crit l'entier n en utilisant le codage bit  bit :
 *  - nombre de bits ncessaires  son enregistrement cod de faon unaire
 *  - 0
 *  - entier n
 */
int ecrirebitabit(FileBAB * fichier, unsigned int n)
{
    if ( fichier->mode[0] != 'w' )
	return 0;

    int i, nb = nbbits(n);
    
    /* On crit d'abord de faon unaire, le nombre de bits ncessaires */
    for ( i=0 ; i < nb ; i++ )
	if ( ! ecrirebit(fichier, 1) )
	    return 0;
    /* Sparateur */
    if ( ! ecrirebit(fichier, 0) )
	return 0;

    /* Bits */
    for ( i=0 ; i < nb ; i++ )
	if ( (n & ( 1 << i )) ) {
	    if ( ! ecrirebit(fichier, 1) )
		return 0;
	} else {
	    if ( ! ecrirebit(fichier, 0) )
		return 0;
	}

    return 1;
}

/* Lit un entier et le place dans n en utilisant le codage bit  bit
 * ( idem  la fonction ecrirebitabit )
 */
int lecturebitabit(FileBAB * fichier, unsigned int * n)
{
    if ( fichier->mode[0] != 'r' )
	return 0;

    int i,nb = 0;
    char bit = 0;

    do {
	if ( ! lirebit(fichier, &bit) )
	    return 0;
	if ( bit )
	    nb++;
    } while (bit);

    *n = 0;
    for ( i=0 ; i < nb ; i++ ) {
	if ( ! lirebit(fichier, &bit) )
	    return 0;
	if ( bit )
	    *n |= ( 1 << i );
    }

    return 1;
}

/*
 * crit un entier n en utilisant le codage octet par octet
 * On crit par paquet de 7 bits et un huitime indique si l'criture se
 * poursuit sur l'octet suivant ou non.
 */
int ecrireoctetparoctet(FileBAB * fichier, unsigned int n)
{
    if ( fichier->mode[0] != 'w' )
	return 0;

    int nboctets = nbbits(n)/7+1; /* Nombre d'octets  crire */
    int bit = 0;
    int i;

    for ( i=1 ; i <= nboctets ; i++ ) {
	/* On crit le bit indiquant la poursuite ou non de l'criture
	 * dans l'octet suivant
	 */
	if ( ! ecrirebit(fichier, ((i == nboctets)?0:1)) )
	    return 0;
	/* On crit 7 bits de l'entier
	 */
	for ( ; bit < 7*i ; bit++ )
	    if ( (n & ( 1 << bit )) ) {
		if ( ! ecrirebit(fichier, 1) )
		    return 0;
	    } else {
		if ( ! ecrirebit(fichier, 0) )
		    return 0;
	    }
    }

    return 1;
}

/*
 * Lit un entier en utilisant le codage octet par octet ( cf au dessus ) 
 */
int lectureoctetparoctet(FileBAB * fichier, unsigned int * n)
{
    if ( fichier->mode[0] != 'r' )
	return 0;

    int lirebits = 1, octet = 1, i; 
    char bit;

    *n = 0;
    while ( lirebits ) {
	/* On regarde si la lecture se poursuivra dans l'octet suivant */
	if ( ! lirebit(fichier, &bit) )
	    return 0;
	if ( bit == 0 )
	    lirebits = 0;
	/* On lit les 7 bits de cet octet */
	for ( i=7*(octet-1) ; i < 7*octet ; i++ ) {
	    if ( ! lirebit(fichier, &bit) )
		return 0;
	    if ( bit )
		*n |= ( 1 << i );
	}
	/* Passage  l'octet suivant */
	octet++;
    }

    return 1;
}
