#include <liste.h>

/* 
 * Alloue de la mmoire pour stocker mot dans une cellule de liste chaine
 */
cellule * creer_cellule(char * mot)
{
    cellule * tmp = NULL;
    
    tmp = malloc(sizeof(cellule));
    if ( tmp == NULL )
	return NULL;

    tmp->mot = malloc(strlen(mot)+1);
    if ( tmp->mot == NULL ) {
	free(tmp);
	return NULL;
    }

    strcpy(tmp->mot, mot);
    tmp->suiv = NULL;

    return tmp;
}

/* 
 * Dtruit la liste chaine pointe
 */
void detruire_liste(liste l)
{
    cellule * tmp;

    while ( l != NULL ) {
	tmp = l;
	l = l->suiv;
	free(tmp->mot);
	free(tmp);
    }
}

/*
 * Ajoute mot  la fin de la liste l
 */
cellule * ajouter_fin_liste(liste * l, char * mot)
{
    cellule * c = creer_cellule(mot);
    if ( c == NULL )
	return NULL;

    if ( *l == NULL ) {
	*l = c;
    } else {
	cellule * tmp = *l;
	while ( tmp->suiv != NULL )
	    tmp = tmp->suiv;
	tmp->suiv = c;
    }

    return c;
}


/*
 * Ajoute un caractre  la fin de la liste l ( de caractres ! )
 */
cellule * ajouter_fin_liste_c(liste * l, char mot)
{
    cellule * c = malloc(sizeof(cellule));
    if ( c == NULL )
	return NULL;

    c->mot = malloc(1);
    if ( c->mot == NULL ) {
	free(c);
	return NULL;
    }

    c->mot[0] = mot;
    c->suiv = NULL;

    if ( *l == NULL ) {
	*l = c;
    } else {
	cellule * tmp = *l;
	while ( tmp->suiv != NULL )
	    tmp = tmp->suiv;
	tmp->suiv = c;
    }

    return c;
}

/* 
 * Dplace la cellule c en tte de la liste l
 */
void met_en_tete(liste * l, cellule * c)
{
    if ( c == NULL || *l == NULL || *l == c )
	return; 

    cellule * prec = *l, * suiv = c->suiv;

    while ( prec->suiv != c )
	prec = prec->suiv;

    c->suiv = *l;
    *l = c;

    prec->suiv = suiv;
}

/* 
 * Retourne le mot dans la liste et met sa position dans *position
 * Note : position = 1 : correspond au premier mot de la liste ( l->mot )
 *        position = 0 : correspond donc  l'absence du mot dans la liste.
 */
cellule * recherche(liste l, char * mot, int * position)
{
    if ( l == NULL )
	return NULL;

    *position = 1;

    while ( l != NULL ) {
	if ( strcmp(mot, l->mot) == 0 )
	    return l;
	(*position)++;
	l = l->suiv;
    }

    *position = 0;

    return NULL;
}


/* 
 * Retourne le caractre dans la liste ( de caractres ! ) et met sa position 
 * dans *position
 * Note : position = 1 : correspond au premier mot de la liste ( l->mot )
 *        position = 0 : correspond donc  l'absence du mot dans la liste.
 */
cellule * recherche_c(liste l, char mot, int * position)
{
    if ( l == NULL )
	return NULL;

    *position = 1;

    while ( l != NULL ) {
	if ( mot == l->mot[0] )
	    return l;
	(*position)++;
	l = l->suiv;
    }

    *position = 0;

    return NULL;
}

/* 
 * Retourne la cellule en position n
 */
cellule * position(liste l, int n)
{
    if ( n < 1 )
	return NULL;

    while ( l != NULL && n > 1 ) {
	n--;
	l = l->suiv;
    }

    return l;
}

void afficher_liste(liste l)
{
    while ( l != NULL ) {
	puts(l->mot);
	l = l->suiv;
    }
}

void afficher_liste_c(liste l)
{
    while ( l != NULL ) {
	putchar(l->mot[0]); putchar('\n');
	l = l->suiv;
    }
}
